pub trait IDatabase {
    async fn init_schema(self: &Self);
}

pub mod sqlite {
    use super::IDatabase;

    type Pool = r2d2::Pool<r2d2_sqlite::SqliteConnectionManager>;
    type Connection = r2d2::PooledConnection<r2d2_sqlite::SqliteConnectionManager>;

    pub struct Database {
        pool: Pool,
    }

    impl Database {
        pub fn connect(file: &str) -> Database {
            let manager = r2d2_sqlite::SqliteConnectionManager::file(file);
            let pool = Pool::new(manager).unwrap();
            Database { pool }
        }
    }

    impl IDatabase for Database {
        async fn init_schema(self: &Self) {
            let conn = self.pool.get().unwrap();
            // TODO: check if tables not exist, otherwise return
            conn.execute_batch(include_str!("../sql/sqlite_schema.sql"))
                .unwrap();
        }
    }
}

pub mod postgres {
    use super::IDatabase;

    type Pool = deadpool_postgres::Pool;

    pub struct Database {
        pool: Pool,
    }

    impl Database {
        pub fn connect(url: &str) -> Database {
            let pool = deadpool_postgres::Config::new() // TODO: config
                .create_pool(
                    Some(deadpool_postgres::Runtime::Tokio1),
                    tokio_postgres::NoTls,
                )
                .unwrap();
            Database { pool }
        }
    }

    impl IDatabase for Database {
        async fn init_schema(self: &Self) {
            let conn = self.pool.get().await.unwrap();
            // TODO: check
            conn.batch_execute(include_str!("../sql/postgres_schema.sql"))
                .await
                .unwrap();
        }
    }
}
